<?php
require __DIR__.'/../generated/vendor/autoload.php';
require __DIR__.'/WebSocketClient.php';
// 包含所有需要的 protobuf 生成类（从上级的 generated 目录）
require __DIR__.'/../generated/Common_pack/ReqLogin.php';
require __DIR__.'/../generated/Common_pack/ResLogin.php';
require __DIR__.'/../generated/Common_pack/EnterHall.php';
require __DIR__.'/../generated/Common_pack/ResEnterHall.php';

use Common_pack\EnterHall;
use Common_pack\ReqLogin;
use Common_pack\ResEnterHall;
use Common_pack\ResLogin;

class GameClient {
    private ?BinaryWebSocketClient $client = null;
    private ?string $userId = null;
    private ?string $nikeName = null;

    public function start() {
        $this->login();
    }

    private function login() {
        $loginReq = new ReqLogin();
        $loginReq->setAccount("xy1");
        $loginReq->setPassword('123');
        
        $binaryData = $loginReq->serializeToString();
        
        $options = [
            'http' => [
                'header'  => [
                    'Content-Type: application/x-protobuf',
                    'X-Protobuf-Schema: common.proto',
                    'X-Protobuf-Message: common_pack.ReqLogin'
                ],
                'method'  => 'POST',
                'content' => $binaryData,
                'ignore_errors' => true,
                'timeout' => 10
            ]
        ];
        
        $context = stream_context_create($options);

        try {
            $response = file_get_contents('http://120.76.130.29:8080/RegLogin', false, $context);

            if ($response !== false) {
                $userResponse = new ResLogin(); 
                $userResponse->mergeFromString($response);
                
                if ($userResponse->hasErrMsg()) {
                    throw new Exception("登录失败: ".$userResponse->getErrMsg()->getMessage());
                }

                echo "登录成功!\n";
                echo "用户ID: ".$userResponse->getUserId()."\n";
                echo "昵称: ".$userResponse->getNikeName()."\n";
                
                $this->userId = $userResponse->getUserId();
                $this->nikeName = $userResponse->getNikeName();

                // 登录成功后才建立WebSocket连接
                $this->initWebSocketConnection();
                
                if ($userResponse->hasUserInfo()) {
                    $this->enterHall($userResponse->getUserInfo()->getUserId());
                }
            } else {
                throw new Exception("请求失败: ".error_get_last()['message']);
            }
        } catch (Exception $e) {
            echo $e->getMessage()."\n";
            exit(1);
        }
    }

    private function initWebSocketConnection() {
        // 初始化WebSocket客户端
        $this->client = new BinaryWebSocketClient('ws://game.example.com:8080');
        
        // 注册消息处理器
        $this->client->on(MsgID::RESENTERHALL, function(ResEnterHall $response) {
            echo "成功进入大厅!\n";
            echo "大厅玩家数: ".$response->getSuccess()."\n";
        });

        // 开始监听消息
        $this->client->startListening();
    }

    private function enterHall($userId) {
        if (!$this->client) {
            throw new Exception("WebSocket连接未建立");
        }

        $enterHall = new EnterHall();
        $enterHall->setUserId($userId);
        
        $this->client->sendBinary(MsgID::ENTERHALL, $enterHall);
        echo "已发送进入大厅请求\n";
    }
}

// 启动客户端
$gameClient = new GameClient();
$gameClient->start();